package com.me.asteroids.systems;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.EntityFactory;
import com.me.asteroids.Utils;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

import static com.me.asteroids.Constants.rand;
import static com.me.asteroids.Components.ACCELERATION;
import static com.me.asteroids.Components.MODEL;
import static com.me.asteroids.Components.PLAYER;
import static com.me.asteroids.Components.POSITION;
import static com.me.asteroids.Components.VELOCITY;

public class PlayerInputSystem extends EntitySystem {

    public Vector2 tmp = new Vector2(0, 1);

    public PlayerInputSystem(Engine engine) {
        super(
                engine,
                PositionComponent.class,
                VelocityComponent.class,
                AccelerationComponent.class,
                PlayerComponent.class
        );
    }

    @Override
    public void processEntity(Entity entity, float dt) {
        PositionComponent positionComponent = POSITION.get(entity);
        VelocityComponent velocityComponent = VELOCITY.get(entity);
        AccelerationComponent accelComponent = ACCELERATION.get(entity);
        PlayerComponent playerComponent = PLAYER.get(entity);

        boolean isLeftPressed = Gdx.input.isKeyPressed(Input.Keys.A);
        boolean isRightPressed = Gdx.input.isKeyPressed(Input.Keys.D);
        if (isLeftPressed && !isRightPressed) {
            velocityComponent.angularVelocity = 300;
        } else if (isRightPressed && !isLeftPressed) {
            velocityComponent.angularVelocity = -300;
        } else {
            velocityComponent.angularVelocity = 0;
        }

        Vector2 acceleration = accelComponent.acceleration;
        Vector2 velocity = velocityComponent.velocity;

        MODEL.get(playerComponent.afterBurner).model.setPosition(positionComponent.position);

        if (Gdx.input.isKeyPressed(Input.Keys.W)) {
            acceleration.set(Utils.setUnitVectorAngle(tmp, positionComponent.rotation).scl(12.5f));

            if (rand.nextFloat() < 0.85) {
                PositionComponent afterBurnerPosition = POSITION.get(playerComponent.afterBurner);
                afterBurnerPosition.position.set(positionComponent.position);
                afterBurnerPosition.rotation = positionComponent.rotation;
                playerComponent.afterBurner.activate();
            } else {
                playerComponent.afterBurner.deactivate();
            }
        } else {
            playerComponent.afterBurner.deactivate();
            if (!velocity.isZero(1f)) {
                acceleration.set(Utils.setUnitVectorAngleRad(tmp, velocity.angleRad()).scl(-2.5f));
            } else {
                acceleration.set(0, 0);
            }
        }

        if (Gdx.input.isKeyJustPressed(Input.Keys.SPACE)) {
            Entity bullet = EntityFactory.createBullet(engine, entity);
            bullet.activate();
        }
    }

}
