package com.me.common.ecs;

import java.util.HashMap;
import java.util.Map;

public final class ComponentMapper<T extends Component> {

    private static final Map<Class<? extends Component>, ComponentMapper> mappers = new HashMap<>();

    private final ComponentType type;
    private final Class<T> typeClass;

    private ComponentMapper(Class<T> typeClass) {
        this.type = ComponentType.getComponentType(typeClass);
        this.typeClass = typeClass;
    }

    public T get(Entity entity) {
        return typeClass.cast(entity.getComponent(type));
    }

    public boolean has(Entity entity) {
        return type.isInBits(entity.componentBits);
    }

    @SuppressWarnings("unchecked")
    public static <T extends Component> ComponentMapper<T> getFor(Class<T> typeClass) {
        ComponentMapper<T> mapper = mappers.get(typeClass);
        if (mapper == null) {
            mapper = new ComponentMapper<>(typeClass);
            mappers.put(typeClass, mapper);
        }
        return mapper;
    }

}
